<?php

if(!defined('PLX_ROOT')) { exit; }

// plxToken::validateFormToken();
if($_SERVER['REQUEST_METHOD'] === 'POST') {
	$key = filter_input(
		INPUT_POST, 'token',
		FILTER_VALIDATE_REGEXP,
		array(
			'options' => array(
				'regexp' => '#\w{' . $plxPlugin::TOKEN_LENGTH . '}#'
			)
		)
	); // $plxPlugin::TOKEN_LENGTH
	if(
		! is_string($key) or
		empty($_SESSION['formtoken']) or
		empty($_SESSION['formtoken'][$key]) or
		$_SESSION['formtoken'][$key] < time()
	) {
		unset($_SESSION['formtoken']);
		die('Security error : invalid or expired token');
	}

	unset($_SESSION['formtoken'][$key]);
}

if(empty($_SESSION[$plugin])) {
	switch($plxPlugin->default_lang) {
		case 'fr' : $preferred = 'nl'; break;
		case 'oc' : $preferred = 'fr'; break;
		case 'es' : $preferred = 'pt'; break;
		case 'pt' : $preferred = 'es'; break;
		case 'ro' : $preferred = 'fr'; break;
		case 'pl' :
		case 'ru' : $preferred = 'de'; break;
		default:
			$preferred = '';
	}
	$_SESSION[$plugin] = array(
		'selection' => $plugin,
		'preferred' => $preferred,
		'translate' => $plxPlugin->default_lang
	);
}
if(empty($_SESSION[$plugin])) {
	$_SESSION[$plugin]['selection'] = $plugin;
}

$plugins = array_keys($plxAdmin->plxPlugins->aPlugins);
sort($plugins);

$langs = array_map(
	function($item) {
		return substr($item, -2);
	},
	glob(PLX_CORE . 'lang/*', GLOB_ONLYDIR)
);

/* -------- first form ------- */
if(filter_has_var(INPUT_POST, 'plugin')) {
	$value = filter_input(INPUT_POST, 'plugin', FILTER_SANITIZE_STRING);
	if(is_string($value) and in_array($value, $plugins)) {
		$_SESSION[$plugin]['selection'] = $value;
	}

	$value = filter_input(INPUT_POST, 'lang', FILTER_SANITIZE_STRING);
	if(is_string($value)) {
		if(in_array($value, $langs)) {
			$_SESSION[$plugin]['preferred'] = $value;
		} else {
			unset($_SESSION[$plugin]['preferred']);
		}
	}
	unset($_SESSION[$plugin]['translate']);
	header('Location: plugin.php?p=' . $plugin);
	exit;
}

$disabled = '';
// $folder = dirname(__DIR__) . '/' . $_SESSION[$plugin]['selection'] . '/lang';
$folder = PLX_PLUGINS . $_SESSION[$plugin]['selection'] . '/lang';
if((!is_dir($folder) and !mkdir($folder)) or !is_writable($folder)) {
	plxMsg::Error(sprintf($plxPlugin->getLang('UNWRITABLE'), $folder));
	$disabled = 'disabled';
}

/* -------- second form ------- */
if(filter_has_var(INPUT_POST, 'translate')) {
	if(
		filter_has_var(INPUT_POST, 'idioms') and
		filter_has_var(INPUT_POST, 'translations') and
		filter_has_var(INPUT_POST, 'translate')
	) {
		$idioms = filter_input(INPUT_POST, 'idioms', FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
		$datas = filter_input(INPUT_POST, 'translations', FILTER_SANITIZE_STRING, FILTER_REQUIRE_ARRAY);
		$translate = filter_input(INPUT_POST, 'translate', FILTER_SANITIZE_STRING);
		if(is_array($idioms) and is_array($datas) and is_string($translate)) {
			$idioms = array_map(
				function ($item) {
					return strtoupper($item);
				},
				$idioms
			);

			$_SESSION[$plugin]['translate'] = (in_array($translate, $langs)) ? $translate : '';

			foreach($datas as $lang=>$translations) {
				// Translations for one or more languages ?
				if(
					!in_array($lang, $langs) or
					($translate != 'all') and $translate != $lang
				) {
					continue;
				}

				$dict = array();
				for($i=0, $iMax=count($idioms); $i<$iMax; $i++) {
					if(empty(trim($idioms[$i]))) { continue; }
					$value = trim($translations[$i]);
					$dict[$idioms[$i]] = addslashes($value);
				}

				if(empty($dict)) { continue; }

				ksort($dict);
				$lines = array_map(
					function($key, $value) {
						return <<< LINE
	'$key'	=> '$value'
LINE;
					},
					array_keys($dict),
					array_values($dict)
				);

				$filename = "$folder/$lang.php";
				if(file_put_contents(
					$filename,
					$plxPlugin::HEAD . PHP_EOL .
					implode(',' . PHP_EOL, $lines) . PHP_EOL .
					$plxPlugin::TAIL . PHP_EOL
				) !== false) {
					plxMsg::Info(L_SAVE_SUCCESSFUL);
				}
			}
		}

		header('Location: plugin.php?p=' . $plugin);
		exit;
	}
}

/* -------------- We have no $_POST -------------- */
$token = plxUtils::charAleatoire($plxPlugin::TOKEN_LENGTH);
$_SESSION['formtoken'][$token] = time() + 3600; // 3600 seconds

$default = $plxPlugin->default_lang;
$preferred = (array_key_exists('preferred', $_SESSION[$plugin])) ? $_SESSION[$plugin]['preferred'] : '';

usort($langs, function($a, $b) use($default, $preferred) {
	foreach(array('en', $default, $preferred) as $k) {
		if($a == $k) return -1;
		if($b == $k) return 1;
	}
	if($a == $b) return 0;
	return strcmp($a, $b);
});

$plxPlugin->get_all_idioms($langs, $_SESSION[$plugin]['selection']);
$idioms = $plxPlugin->idioms;

$id = 'translate_all';
$rowId = $plugin . '-title-cols';
$contentId = $plugin . '-content';
$actionId =  $plugin . '-action';
?>
<form method="post" class="<?= $plugin ?>">
	<div class="in-action-bar">
		<label for="id_plugin"><?= $plxPlugin->getLang('PLUGIN') ?></label>
		<select name="plugin" id="id_plugin">
<?php
foreach($plugins as $p) {
	$selected = ($p == $_SESSION[$plugin]['selection']) ? 'selected' : '';
?>
			<option <?= $selected ?>><?= basename($p) ?></option>
<?php
}
?>
		</select>
		<label for="id_lang"><?= $plxPlugin->getLang('3RD_LANG'); ?></label>
		<select name="lang" id="id_lang">
<?php
foreach($langs as $p) {
	$selected = ($p == $preferred) ? 'selected' : '';
	if($p == 'en' or $p == $plxPlugin->default_lang) { continue; }
?>
			<option value="<?= $p ?>" <?= $selected ?>><?= $p ?></option>
<?php
}
?>
		</select>
		<input name="token" value="<?= $token ?>" type="hidden" />
		<input type="submit" value="<?= $plxPlugin->getLang('VALIDATE') ?>" />
	</div>
</form>

<form method="post" class="<?= $plugin ?>" name="<?= $plugin ?>_grid">
	<div class="scrollable-table">
		<table class="<?= $plugin ?>-table">
			<thead>
				<tr id="<?= $rowId ?>">
					<th><input type="radio" name="translate" id="<?= $id ?>" value="all"><label for="<?= $id ?>"><?= $plxPlugin->getLang('ALL_LANGS') ?></label></th>
<?php
if(!empty($_SESSION[$plugin]['translate'])) {
	$l = $_SESSION[$plugin]['translate'];
} elseif(!empty($preferred)) {
	$l = $preferred;
} else {
	$l = $plxPlugin->default_lang;
}
// $l = (!empty($_SESSION[$plugin]['translate'])) ? $_SESSION[$plugin]['translate'] : (!empty($preferred)) ? $preferred : $plxPlugin->default_lang;
foreach($langs as $lang) {
	$id = "translate_$lang";
	$checked = ($lang == $l) ? 'checked' : '';
	$className = ($lang == $l) ? 'class="selected"' : '';
?>
					<th <?= $className ?>><input type="radio" name="translate" id="<?= $id ?>" value="<?= $lang ?>" <?= $checked ?>><label for="<?= $id ?>"><?= $plxPlugin->getLang('L_' . strtoupper($lang)) ?></label></th>
<?php
}
?>
				</tr>
			</thead>
			<tbody id="<?= $contentId ?>">
<?php
$autofocus = '';
$row = kzTranslate::TABINDEX_START;
foreach($idioms as $idiom=>$translations) {
?>
				<tr>
					<td><input type="text" name="idioms[]" value="<?= $idiom ?>" placeholder="<?= $plxPlugin->getLang('NEW_IDIOM') ?>" /></td>
<?php
	$index = $row;
	foreach($langs as $lang) {
		if($lang == $plxPlugin->default_lang and empty($autofocus)) {
			$autofocus = ' autofocus';
		}
		$index += kzTranslate::COL_STEP;
		$value = (!empty($translations[$lang])) ? stripslashes($translations[$lang]) : '';
?>
					<td><input type="text" name="translations[<?= $lang ?>][]" value="<?= $value ?>" tabindex="<?= $index ?>"<?= $autofocus ?>/></td>
<?php
	}
?>
				</tr>
<?php
	$row++;
}
/* ---------- More 5 idioms ----------- */
for($i=0, $iMax=5; $i<$iMax; $i++) {
?>
				<tr>
					<td><input type="text" name="idioms[]" placeholder="<?= $plxPlugin->getLang('NEW_IDIOM') ?>" /></td>
<?php
	$index = $row * (count($langs) + 1);
	foreach($langs as $lang) {
		$index += kzTranslate::COL_STEP;
?>
					<td><input type="text" name="translations[<?= $lang ?>][]" value="" tabindex="<?= $index ?>" /></td>
<?php
	}
?>
				</tr>
<?php
	$row++;
}
?>
			</tbody>
		</table>
	</div>
	<div class="in-action-bar" id="<?= $actionId ?>">
		<input name="token" value="<?= $token ?>" type="hidden" />
		<input type="button" id="id_translate" value="<?= $plxPlugin->getLang('TRANSLATE') ?>" title="<?= $plxPlugin->getLang('TRANSLATE_TITLE') ?>" />
		<input type="submit" value="<?= $plxPlugin->getLang('SAVE') ?>" <?= $disabled ?> />
		<p><?= $plxPlugin->getLang('CTRL_CLICK') ?></p>
	</div>
</form>
<div class="<?= $plugin ?> logos">
	<a href="https://mymemory.translated.net/" target="_blank"><img src="https://mymemory.translated.net/public/img/mym_logo_horizontal.svg" alt="MyMemory" /></a>
	<a id="linguee" href="https://www.linguee.com/" data-urlbase="https://www.linguee.com/#1-#2/search?query=#3" target="_blank"><img src="https://d1wigddrwdtsce.cloudfront.net/img5/dl_Linguee_Logo.svg" alt="Linguee" /></a>
	<a href="https://translate.google.com/" target="_blank"><img src="https://www.gstatic.com/images/branding/googlelogo/svg/googlelogo_clr_74x24px.svg" alt="Google" /></a>
	<a href="https://translate.yandex.com/" target="_blank"><img src="https://upload.wikimedia.org/wikipedia/commons/9/91/Yandex_logo_en.svg" alt="Yandex" /></a>
</div>
<div id="<?= $plugin ?>-spinner">
	<div class="bounce1"></div>
	<div class="bounce2"></div>
	<div class="bounce3"></div>
</div>

<script type="text/javascript">
	(function() {
		'use strict';
<?php
	$all_languages = '\'' . implode('\', \'', $langs) . '\'';
	// For translating one cell only
	$engine = 'google';
	switch($engine) {
		case 'google' :
			$template = kzTranslate::GOOGLE_TRANSLATION;
			$code = <<< 'EOT'
								// this.input.value = data[0][0][0];
								this.input.value = data[0].map(function(item) { return item[0]; }).join('');
								return;
EOT;
			break;
		case 'mymemory' :
			$template = kzTranslate::MY_MEMORY_TRANSLATION;
			$code = <<< 'EOT'
								if('responseStatus' in data && data.responseStatus == 200) {
									this.input.value = data.responseData.translatedText;
									return;
								}
EOT;
			break;
	}
/* "oc" language is unknown at Google's translations */
?>
		const languages = {
			en: 'english',
			de: 'german',
			es: 'spanish',
			fr: 'french',
			it: 'italian',
			nl: 'dutch',
			pl: 'polish',
			pt: 'portuguese',
			ro: 'romanian',
			ru: 'russian'
		}
		const ALL_LANGUAGES = [<?= $all_languages ?>];
		const linguee = document.getElementById('linguee');

		const SELECTED = 'selected';
		const translateBtn = document.getElementById('id_translate');
		const content = document.getElementById('<?= $contentId ?>'); // tbody
		const columnTitles = document.getElementById('<?= $rowId ?>'); // thead
		const spinner = document.getElementById('<?= $plugin ?>-spinner');

		function setColumnsWidth(selectedLang, clickedCell) {
			for(var i=0, iMax=columnTitles.cells.length; i<iMax; i++) {
				columnTitles.cells[i].classList.remove(SELECTED);
			}
			if(selectedLang != '' && selectedLang != 'all') {
				const index = ALL_LANGUAGES.indexOf(selectedLang)
				if(index < 0) { return; }
				columnTitles.cells[index+1].classList.add(SELECTED);

				const f = (selectedLang == 'oc');
				translateBtn.disabled = f;
				const el = document.querySelector('#<?= $plugin ?>-action p:last-of-type');
				if(el != null) {
					if(f) {
						el.classList.add('disabled');
					} else {
						el.classList.remove('disabled');
					}
				}

				if(typeof clickedCell == 'undefined') {
					content.rows[0].cells[index+1].firstChild.focus();
				}
			}
		}

		// the selected language is changed
		columnTitles.addEventListener('change', function(event) {
			if(event.target.name == 'translate') {
				event.preventDefault();
				setColumnsWidth(event.target.value);
			}
		});

		// Translation for one cell only by clicking on the mouse
		content.addEventListener('click', function(event) {
			if(event.target.tagName != 'INPUT' || !(event.shiftKey || event.ctrlKey)) { return; }

			event.preventDefault();
			const lang = event.target.name.replace(/^translations\[([a-z]{2})\].*/, '$1');
			if(ALL_LANGUAGES.indexOf(lang) >= 0 && lang != 'oc') {
				// Looking for a source language
				const rowIndex = event.target.tabIndex % <?= kzTranslate::COL_STEP ?> - <?= kzTranslate::TABINDEX_START ?>;
				var value;
				var j = -1;
				for(let i=0, iMax=ALL_LANGUAGES.length; i<iMax; i++) {
					if(ALL_LANGUAGES[i] == lang) { continue; }
					const k = 'translations[' + ALL_LANGUAGES[i] + '][]';
					const input = document.forms.<?= $plugin ?>_grid.elements[k][rowIndex];
					const v = input.value.trim();
					if(v.length > 2) {
						j = i;
						value = v;
						break;
					}
				}
				if(j < 0) {
					console.error('No segment source for translation');
					return;
				}


				if(event.shiftKey) {
					// shift key is pressed in the same time : translation from Linguee of a new tab
					const source = ALL_LANGUAGES[j];
					if(source in languages && lang in languages) {
						linguee.href = linguee.dataset.urlbase.replace(/#(\d+)/g, function(match, index) {
							switch(index) {
								case '1': return languages[source]; break; // source
								case '2': return languages[lang]; break; // target
								case '3': return encodeURI(value); break; // idiom
							}
						});
						// console.log(linguee.href);
						linguee.click();
						return;
					}

					console.error('unknown "' + lang + '" language from Linguee');
					return;
				}

				if(event.ctrlKey) {
					// Translate one cell
					const url = '<?= $template ?>'.replace(/#(\d+)/g, function(match, index) {
						switch(index) {
							case '1': return ALL_LANGUAGES[j]; break;
							case '2': return lang; break;
							case '3': return encodeURI(value); break;
						}
					});
					console.log(url);
					const xhr = new XMLHttpRequest();
					xhr.onload = function(evnt) {
						spinner.classList.remove('active');
						if(this.input) {
							if(this.getResponseHeader('Content-Type').startsWith('application/json')) {
								const data = JSON.parse(this.responseText);
<?= $code ?>
							}
							console.error(this.responseText);
						}
					}
					xhr.onerror = function(event) {
						console.error(this.status, this.statusText, this.responseUrl);
						spinner.classList.remove('active');
					}
					xhr.input = event.target;
					xhr.open('GET', url);
					xhr.send();
					spinner.classList.add('active');
					return;
				}
			}

			console.error('No automatic translation for "' + lang + '" language');
		});

		content.addEventListener('focusin', function(event) {
			if(event.target.tagName == 'INPUT') {
				const lang = event.target.name.replace(/^translations\[(\w{2})\].*/, '$1');
				const previousLang = document.forms.<?= $plugin ?>_grid.elements['translate'].value;
				if(previousLang != lang) {
					document.forms.<?= $plugin ?>_grid.elements['translate'].value = lang;
					setColumnsWidth(lang, true);
				}
			}
		});

		function scanMissingTranslations(source, target) {
			const nameSource = 'translations[' + source + '][]';
			const nameTarget = 'translations[' + target + '][]';
			const lines = [];
			const elements = document.forms.<?= $plugin ?>_grid.elements;
			for(var i=0, iMax = elements[nameTarget].length; i<iMax; i++) {
				const value = elements[nameSource][i].value.trim();
				if(value.length < 2 || elements[nameTarget][i].value.trim().length > 2) {
					continue;
				}
				lines.push(i + ':' + value);
			}
			return lines;
		}

		// Translate every empty cell from one or more languages from Google
		if(translateBtn != null) {
			translateBtn.addEventListener('click', function(event) {
				event.preventDefault();
				var target = document.forms.kzTranslate_grid.elements.translate.value;
				if(target == 'oc') { return; }

				// Look for a source language for the query ( référence langue )
				var source;
				if(target == ALL_LANGUAGES[0]) {
					source = ALL_LANGUAGES[1];
				} else if(target == ALL_LANGUAGES[1]) {
					source = ALL_LANGUAGES[0];
				} else {
					const counts = [0, 0];
					for(var i=0, iMax=<?= count($idioms) ?>; i<iMax; i++); {
						const row = content.rows[i];
						for(var j=0; j<2; j++) {
							const input = row.cells[j+1].querySelector('input');
							const value = input.value.trim();
							if(value.length > 2) {
								counts[j]++;
							}
						}
					}
					source = ALL_LANGUAGES[(counts[0] >= counts[1]) ? 0 : 1];
				}

				// console.log('[<?= $plugin ?>] Source language : ', source);
				const xhr = new XMLHttpRequest();
				xhr.process = function() {
					const source = this.source;
					var target = this.target;
					var lines = null;
					if(this.index >= 0 || this.target == 'all') {
						while(this.index < ALL_LANGUAGES.length) {
							target = ALL_LANGUAGES[this.index];
							this.index++;
							if(target != 'oc' && target != source) {
								lines = scanMissingTranslations(source, target);
								if(lines.length > 1) {
									this.target = target;
									break;
								}
							}
						}
					} else {
						lines = scanMissingTranslations(source, target);
					}

					if(lines == null || lines.length <= 1) {
						spinner.classList.remove('active');
						return;
					}

					const url = '<?= kzTranslate::GOOGLE_TRANSLATION ?>'.replace(/#(\d+)/g, function(match, index) {
						switch(index) {
							case '1': return source; break;
							case '2': return target; break;
							case '3': return encodeURIComponent(lines.join('\n')); break;
						}
					});
					// console.log(url);
					this.open('GET', url);
					this.send();
					spinner.classList.add('active');
				}

				xhr.onerror = function(event) {
					console.error(this.status, this.statusText);
				}

				xhr.onload = function() {
					if(this.getResponseHeader('Content-Type').startsWith('application/json')) {
						const lang = this.target;
						const elements = document.forms.kzTranslate_grid.elements['translations[' + lang + '][]'];
						const datas = JSON.parse(this.responseText);
						// Google breaks each source line at each '.', '!', '?', ...
						const lines = datas[0].map(function(item) { return item[0]; }).join('').split('\n');
						lines.forEach(function(item) {
							const matches = /^(\d+):(.*)$/.exec(item.trim());
							if(matches != null) {
								elements[matches[1]].value = matches[2];
							} else {
								console.error(item.trim());
							}
						});
						if(this.index > 0 && this.index < ALL_LANGUAGES.length -1) {
							this.process();
						} else {
							spinner.classList.remove('active');
						}
						return;
					}
					console.error('Bad Content-Type');
				}
				xhr.source = source;
				xhr.target = target;
				xhr.index = (target == 'all') ? 0 : -1;
				xhr.process();
			});
		}

		/* Hack against PluXml : bad stylesheet on large screen ! Set fixed width on the left side panel */
		document.body.classList.add('<?= $plugin ?>');
	})();
</script>
